# Copyright (C) 2024, UChicago Argonne, LLC
# Licensed under the 3-clause BSD license.  See accompanying LICENSE.txt file
# in the top-level directory.

import os
import time
import pytest
import random
from pprint import pformat
from _pytest.assertion import truncate

from PBS_Utils.pbs_instrument import instrument_setup, TracerSimple, ot_decorate_span, get_tracer, TracerFake

truncate.DEFAULT_MAX_LINES = 8192
truncate.DEFAULT_MAX_CHARS = 8192


@pytest.mark.skipif(os.geteuid() == 0, reason="test requires a valid user, not 0")
def test_Tracer_OT_00():
    pytest.importorskip("opentelemetry", reason="opentelemetry not found.")
    output_path = "/tmp/ot_example.jsonl"
    tracer = None

    @ot_decorate_span(tracer)
    def do_something_05(a, b):
        return [random.random() for _ in range(int(a * b))]

    def do_something_06(a, b):
        return [random.random() for _ in range(int(a * b))]

    do_something_05(100, 100)
    do_something_06(100, 100)

    # will not setup a tracer
    with instrument_setup(None, output_path) as tracer:
        do_something_05(100, 100)
        ot_decorate_span(tracer)(do_something_06)(10, 100)

    # do not start another tracer after this.
    with instrument_setup("example", output_path) as tracer:

        @ot_decorate_span(tracer)
        def do_something_00(a: float, b: float):
            return [random.random() for _ in range(int(a * b))]

        def do_something_01(a: float, b: float):
            return [random.random() for _ in range(int(a * b))]

        @ot_decorate_span(tracer)
        def do_something_02():
            raise ValueError("Boom!")

        def do_something_03():
            raise ValueError("Boom!")

        @ot_decorate_span(tracer)
        def do_something_04():
            do_something_00(10, 100)
            do_something_00(100, 100)
            ot_decorate_span(tracer)(do_something_01)(10, 100)
            ot_decorate_span(tracer)(do_something_01)(100, 100)

        do_something_00(10, 100)
        do_something_00(100, 100)
        ot_decorate_span(tracer)(do_something_01)(10, 100)
        ot_decorate_span(tracer)(do_something_01)(100, 100)
        try:
            do_something_02()
        except:
            pass

        try:
            ot_decorate_span(tracer)(do_something_03)()
        except:
            pass

        do_something_04()

    do_something_04()

    assert os.path.exists(output_path)


@pytest.mark.skipif(os.geteuid() == 0, reason="test requires a valid user, not 0")
def test_TracerFake_00():
    pytest.importorskip("opentelemetry", reason="opentelemetry not found.")
    with instrument_setup("test_00", "/tmp/tracer_fake_00.jsonl", TracerFake) as tracer:
        with tracer.start_as_current_span("a"):
            a = 1
        with tracer.start_as_current_span("b"):
            b = a
    assert a == b


# @pytest.importorskip('opentelemetry', reason='opentelemetry not found.')
@pytest.mark.skipif(os.geteuid() == 0, reason="test requires a valid user, not 0")
def test_TracerSimple_01():
    pytest.importorskip("opentelemetry", reason="opentelemetry not found.")
    sleep_time = 0.1
    with instrument_setup("test_01", "/tmp/tracer_simple_01.jsonl", TracerSimple) as tracer:
        with tracer.start_as_current_span("a"):
            time.sleep(sleep_time)
        with tracer.start_as_current_span("b"):
            time.sleep(sleep_time)
    assert len(tracer.traces) == 3
    print(pformat(tracer.traces))
    assert (tracer.td_code - (sleep_time * 2)) < 0.02, tracer.td_code - sleep_time
